<?php
/**
* Plugin Name: Social Listening
* Plugin URI: https://social.santafecap.com
* Description: This pluging searches for keywords you want to monitor across different social media platforms. 
* Version: 1.0
* Author: Santafecap
* Author URI: https://www.santafecap.com
* 
**/

add_action('admin_menu', 'social_listening_admin_menu');
register_activation_hook(__FILE__, 'social_listening_activation');
register_uninstall_hook(__FILE__, 'social_listening_uninstall');
register_deactivation_hook(__FILE__, 'social_listening_deactivation');
add_action( 'init', 'social_listening_register_post_type' );
add_action( 'admin_action_social_listening', 'social_listening_admin_action' );
add_action( 'social_listening_cron_hook', 'social_listening_cron_exec' );
if ( ! wp_next_scheduled( 'social_listening_cron_hook' ) ) {
    wp_schedule_event( time(), 'hourly', 'social_listening_cron_hook' );
}

 
function social_listening_admin_menu(){

    add_submenu_page(
        'edit.php?post_type=social_l_results',
        __( 'Settings', 'textdomain' ),
        __( 'Settings', 'textdomain' ),
        'edit_posts',
        'social-listening-settings',
        'social_listening_config'
    );

    add_submenu_page(
        'edit.php?post_type=social_l_results',
        __( 'Help Center', 'textdomain' ),
        __( 'Help Center', 'textdomain' ),
        'edit_posts',
        'social-listening-help',
        'social_listening_help'
    );

     add_submenu_page(
        'edit.php?post_type=social_l_results',
        __( 'Upgrade Plan', 'textdomain' ),
        __( 'Upgrade Plan', 'textdomain' ),
        'edit_posts',
        'social-listening-upgrade',
        'social_listening_upgrade'
    );

}

function social_listening_register_post_type() {

	$labels = array(
	 'name' => __( 'Social Listening', 'social_listening' ),
	 'singular_name' => __( 'Result', 'social_listening'),
	 'view_item' => __( 'View Result', 'social_listening' ),
	 'search_items' => __( 'Search Results', 'social_listening' ),
	 'not_found' =>  __( 'No Result Found', 'social_listening' ),
	 'not_found_in_trash' => __( 'No Results found in Trash', 'social_listening' ),
	);

	$args = array(
	 'labels' => $labels,
	 'has_archive' => true,
	 'public' => true,
	 'hierarchical' => false,
	 'supports' => array( 'title', 'editor', 'excerpt', 'author', 'thumbnail', 'comments', 'revisions', 'custom-fields', ),
	 'map_meta_cap'        => true,
     'capabilities' => array(
        'create_posts' => 'do_not_allow'
      ),
	 'rewrite'   => array( 'slug' => 'results' ),
	 'show_in_rest' => true
	);

	register_post_type( 'sociaL_l_results', $args );

	add_post_type_support( 'sociaL_l_results', 'author' );
}


function social_listening_config() {

	$apikey = get_option('apikey_social_listening');
	$email = get_option('email_social_listening');
	$search_terms = '';
	$exclusion_terms = '';
	$subscription = '';
	$reddit = 0;
    $twitter = 0;
    $news = 0;
    $instagram = 0;
    $youtube = 0;
    $hackernews = 0;
    $lobste = 0;
    $google = 0;
    $facebook = 0;
    $quora = 0;
    $pinterest = 0;

	if(isset($_POST['submit'])){

		$subscription = $_POST['subscription'];
		if($subscription == 'Free Plan') { $reddit = 1;}

		if(!empty($_POST['reddit'])) {  $reddit = $_POST['reddit']; }
	    if(!empty($_POST['twitter'])) {  $twitter = $_POST['twitter']; }
	    if(!empty($_POST['news'])) {  $news = $_POST['news']; }
	    if(!empty($_POST['instagram'])) {  $instagram = $_POST['instagram']; }
	    if(!empty($_POST['youtube'])) {  $youtube = $_POST['youtube']; }
	    if(!empty($_POST['hackernews'])) {  $hackernews = $_POST['hackernews']; }
	    if(!empty($_POST['lobste'])) {  $lobste = $_POST['lobste']; }
	    if(!empty($_POST['google'])) {  $google = $_POST['google']; }
	    if(!empty($_POST['facebook'])) {  $facebook = $_POST['facebook']; }
	    if(!empty($_POST['quora'])) {  $quora = $_POST['quora']; }
	    if(!empty($_POST['pinterest'])) {  $pinterest = $_POST['pinterest']; }

        //send to firestore and update 
        $body = array(
		'email'   => $email,
		'apikey' => $apikey,
		'searchTerms' => $_POST['social-listening-search_terms'],
		'exclusion' => $_POST['social-listening-exclusion_terms'],
		'reddit' => $reddit,
	    'twitter' => $twitter,
	    'news' => $news,
	    'instagram' => $instagram,
	    'youtube' => $youtube,
	    'hackernews' => $hackernews,
	    'lobste' => $lobste,
	    'google' => $google,
	    'facebook' => $facebook,
	    'quora' => $quora,
	    'pinterest' => $pinterest,
		);

		$args = array(
			'body'        => $body,
			'timeout'     => '5',
			'redirection' => '5',
			'httpversion' => '1.0',
			'blocking'    => true,
			'headers'     => array(),
			'cookies'     => array(),
		);

		$response = wp_remote_post( 'https://updateterms-awm6u4paoq-uc.a.run.app', $args );

		echo ' <div class="notice notice-success is-dismissible">
			      <p>Settings Saved. Please allow 24-48 hours for results to populate.</p>
			  </div>';

	    $search_terms = $_POST['social-listening-search_terms'];
	    $exclusion_terms = $_POST['social-listening-exclusion_terms'];
	   
    }
    else {
    	//connect to firestore, get terms
		$request = wp_remote_get( 'https://getterms-awm6u4paoq-uc.a.run.app?email='.$email.'&apikey='.$apikey );

		if( is_wp_error( $request ) ) {
			echo '<div id="message" class="error"><p>Error getting results, please try again.</p></div>';
			return false;
		}

		$body = wp_remote_retrieve_body( $request );

		$data = json_decode( $body );

		if( ! empty( $data ) ) {
			$search_terms = $data->searchTerms;
			$exclusion_terms = $data->exclusion;
			$subscription = $data->subscription;
			$reddit = $data->reddit;
			$twitter = $data->twitter;
			$news = $data->news;
			$instagram = $data->instagram;
			$youtube = $data->youtube;
			$hackernews = $data->hackernews;
			$lobste = $data->lobste;
			$google = $data->google;
			$facebook = $data->facebook;
			$quora = $data->quora;
			$pinterest = $data->pinterest;
		}
		else {
			echo '<div id="message" class="error"><p>Error getting results, please try again.</p></div>';
		}
    }

    //limit the checkboxes
	$disable_checkboxes = true;
	if($subscription != 'Free Plan') {
		$disable_checkboxes = false;
	}

	$box_limit = 1;
	if($subscription == 'Starter Plan') {
		$box_limit = 3;
	}
	else if($subscription == 'Business Plan') {
		$box_limit = 5;
	}
	else if($subscription == 'Enterprise Plan') {
		$box_limit = 100;
	}
	?>
	<h2>Social Listening Plugin Search Settings</h2>		

	<form action="" method="post" id="social_listening_terms_form" >			

		 <input type="hidden" name="action" value="social_listening" />
		 <input type="hidden" name="subscription" value="<?php echo $subscription?>" />
		<div>
			<br>
			<label>Search Terms: </label><br>
			<input required id="social-listening-search_terms" type="text" name="social-listening-search_terms" value="<?php echo $search_terms; ?>" placeholder="" size="70"/><br>
		</div>
		<div>
			<label>Exclusion Terms:</label><br>
			<input id="social-listening-exclusion_terms" type="text" name="social-listening-exclusion_terms" value="<?php echo $exclusion_terms; ?>" placeholder="" size="70"/><br>
		</div> 

		<div style="margin-top: 30px">
			<label>Social Media Platforms to Search:</label><br>
		</div>

		<div>
			<label>Reddit:</label>
			<input id="reddit" class="platforms" type="checkbox" name="reddit" value="1" <?php if($reddit) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)" /><br>
		</div> 
		<div>
			<label>Twitter/X:</label>
			<input id="twitter" class="platforms" type="checkbox" name="twitter" value="1" <?php if($twitter) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div> 
		<div>
			<label>News/blogs:</label>
			<input id="news" class="platforms" type="checkbox" name="news" value="1" <?php if($news) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div> 
		<!--<div>
			<label>Instagram:</label>
			<input id="instagram" class="platforms" type="checkbox" name="instagram" value="1" <?php if($instagram) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div> -->
		<div>
			<label>Youtube:</label>
			<input id="youtube" class="platforms" type="checkbox" name="youtube" value="1" <?php if($youtube) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div> 
		<div>
			<label>Hacker News:</label>
			<input id="hackernews" class="platforms" type="checkbox" name="hackernews" value="1" <?php if($hackernews) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div>
		<div>
			<label>Lobste.rs:</label>
			<input id="lobste" class="platforms" type="checkbox" name="lobste" value="1" <?php if($lobste) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div>
		<div>
			<label>Google Search:</label>
			<input id="google" class="platforms" type="checkbox" name="google" value="1" <?php if($google) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div>
		<!--<div>
			<label>Facebook:</label>
			<input id="facebook" class="platforms" type="checkbox" name="facebook" value="1" <?php if($facebook) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div>
		<div>
			<label>Quora:</label>
			<input id="quora" class="platforms" type="checkbox" name="quora" value="1" <?php if($quora) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div>
		<div>
			<label>Pinterest:</label>
			<input id="pinterest" class="platforms" type="checkbox" name="pinterest" value="1" <?php if($pinterest) echo "checked" ?> <?php if($disable_checkboxes) echo "disabled" ?> onclick="checkBoxLimit(<?php echo $box_limit;?>)"/><br>
		</div>-->



		<p class="submit"><input type="submit" name="submit" id="submit" class="button button-primary" value="Save Settings"></p>
	</form>

	<div style="width: 30%; padding: 10px; background-color: white; box-shadow: 0 4px 8px 0 rgba(0,0,0,0.2); transition: 0.3s; border-radius: 5px;">You are currently on the <b><?php echo $subscription ?> </b>subscription
	<div style="margin-top:30px;">Your API key is: <?php echo $apikey ?></div>
	</div>

	<div style="margin-top: 50px;"><h3>Please visit the <a href="edit.php?post_type=social_l_results&page=social-listening-help">Help Center</a> for any questions</h3></div>
	<script type="text/javascript">

		    window.addEventListener("load", checkBoxLimit(<?php echo $box_limit;?>));

			
			function checkBoxLimit(boxLimit) {
				var num = document.querySelectorAll('input[type="checkbox"]:checked').length;

				if(num >= boxLimit) {
					var matches = document.querySelectorAll('input[type="checkbox"]:not(:checked)');
					for (let i = 0; i < matches.length; i++) {
					  var inputID = matches[i].id;
					  document.getElementById(inputID).disabled = true;
					}
				}
				else {
					var matches = document.querySelectorAll('input[type="checkbox"]:not(:checked)');

					for (let i = 0; i < matches.length; i++) {
					  var inputID = matches[i].id;
					  document.getElementById(inputID).disabled = false;
					}
				}
		    }
			
	</script>
	<a href="https://social.santafecap.com/tos.html">Terms of Use</a>
	<?php
}

function social_listening_activation() {

	//grab email, generate API key
	$email = get_option('admin_email');
	
	$apikey = getApiKey();

	//store in WP
	update_option('email_social_listening', $email);
	update_option('apikey_social_listening', $apikey);

	//put in firestore
	$body = array(
		'email'   => $email,
		'apikey' => $apikey,
	);

	$args = array(
		'body'        => $body,
		'timeout'     => '5',
		'redirection' => '5',
		'httpversion' => '1.0',
		'blocking'    => true,
		'headers'     => array(),
		'cookies'     => array(),
	);

	$response = wp_remote_post( 'https://initdata-awm6u4paoq-uc.a.run.app', $args );

	if($response == 'User Already Exists') {

		add_action( 'admin_notices', 'user_exist_notice' );

	}

}

function user_exist_notice() {
	?>
  <div class="notice notice-warning is-dismissible">
      <p>This email has been registered already. We will now load the previous settings.</p>
  </div>
  <?php
}


function social_listening_uninstall() {
	
}

function social_listening_deactivation() {
	$timestamp = wp_next_scheduled( 'social_listening_cron_hook' );
    wp_unschedule_event( $timestamp, 'social_listening_cron_hook' );
}


function social_listening_cron_exec() {

	//this is to fetch data to populate custom post type
	$apikey = get_option('apikey_social_listening');
	$email = get_option('email_social_listening');

	//connect to firestore, get terms
	$request = wp_remote_get( 'https://getresults-awm6u4paoq-uc.a.run.app?email='.$email.'&apikey='.$apikey );

	if( is_wp_error( $request ) ) {
		echo '<div id="message" class="error"><p>Error getting results, please try again.</p></div>';
		return false;
	}

	$body = wp_remote_retrieve_body( $request );

	$data = json_decode( $body );

	if( ! empty( $data ) ) {
		// Check if post exist allready in WP
        $existing_posts = get_posts( array('post_type' => 'sociaL_l_results', 'post_status' => 'private', 'numberposts' => -1) );         
        $urls = array();
        foreach($existing_posts as $post) {
           $url = get_post_meta( $post->ID, 'apiurl', true );
           array_push($urls, $url);
        }

        foreach ($data as $item) { 

        	 if(!in_array($item->url, $urls ) && $item->type != 'makassan' && $item->url != '') {

        	 	$content = '<div style="margin:10px">Source: '.$item->source.'</div><div style="margin:10px">Content: '.$item->text.'</div><div style="margin:10px"> URL: '.$item->cleanURL.'</div>';

        	 	$post_title = $item->title;
        	 	if($post_title == '') { $post_title = $item->source; }
	           
	            // New post data object to set as a post
	              $new_post = array(
	                'post_type'     => 'sociaL_l_results',
	                'post_title'    => $post_title,
	                'post_status'   => 'private',
	                'post_author'   => 1,
	                'post_content'  => $content,
	                 'meta_input' => array(
		                'apiurl' => $item->url,
		              )
	              );
	              
	              // Insert the post into the database
	              wp_insert_post($new_post);
	          }

        }
	}


}

function social_listening_upgrade() {
	include 'views/upgrade-plan-view.php';
}

function social_listening_help() {
	include 'views/help-center-view.php';
}

function getApiKey() {
    $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $randomString = '';
 
    for ($i = 0; $i < 20; $i++) {
        $index = rand(0, strlen($characters) - 1);
        $randomString .= $characters[$index];
    }
 
    return $randomString;
}